{ **************************************************************************** }
{ *                                                                          * }
{ * ASTERIX: HUNT FOR THE BOARS (Ludotronic) Simulator                       * }
{ * by Luca "MADrigal" Antignano, September 2023                             * }
{ *                                                                          * }
{ * http://www.madrigaldesign.it/sim/                                        * }
{ * lucantignano@gmail.com                                                   * }
{ *                                                                          * }
{ * This source code (the "Software") is released as "open source" under the * }
{ * "Creative Common Attribution-NonCommercial-ShareAlike" license 4.0       * }
{ * available at:                                                            * }
{ * https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode              * }
{ *                                                                          * }
{ * In addition to the constraints set by the CC license, you must respect   * }
{ * the following rules:                                                     * }
{ * a. do not offer the Software on a pay-per-play basis.                    * }
{ * b. do not distribute the Software through pre-paid access websites or    * }
{ *    BBS's.                                                                * }
{ * c. do not use the Software in publications of any form unless written    * }
{ *    consent is obtained by the author.                                    * }
{ * d. do not distribute the Software through mirror websites, unless        * }
{ *    written consent is obtained by the author.                            * }
{ * e. do not distribute older versions of the Software through authorized   * }
{ *    mirror websites after newer ones have been made available by the      * }
{ *    author.                                                               * }
{ *                                                                          * }
{ * Excerpt from the CC BY-NC-SA license 4.0                                 * }
{ * ========================================                                 * }
{ * BY: You must give appropriate credit, provide a link to the license, and * }
{ *     indicate if changes were made. You may do so in any reasonable       * }
{ *     manner, but not in any way that suggests the licensor endorses you   * }
{ *     or your use.                                                         * }
{ * NC: You may not use the material for commercial purposes.                * }
{ * SA: If you remix, transform, or build upon the material, you must        * }
{ *     distribute your contributions under the same license as the          * }
{ *     original.                                                            * }
{ *                                                                          * }
{ **************************************************************************** }

unit unit1;
{ this unit contains the main form implementation                              }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Math, Registry, Jpeg, Pngimage, Fmod, Fmodtypes, StdCtrls, Menus,
  ShellAPI, {SK} IniFiles;

type
  TForm1 = class(TForm)
{ * HANDHELD GRAPHICS, INCLUDING BUTTONS                                     * }

    // fullscreen mode items
    btn_game_menu_top:        TImage;
    btn_game_menu_background: TImage;

    im_backdrop:              TImage;
    im_backdrop_logo:         TImage;    im_background_screen_top: TImage;

    btn_game_a_down:          TImage;    btn_game_a_top:           TImage;
    btn_game_b_down:          TImage;    btn_game_b_top:           TImage;
    btn_time_down:            TImage;    btn_time_top:             TImage;
                                         btn_set_top:              TImage;

    btn_left_down:            TImage;    btn_left_top:             TImage;
    btn_right_down:           TImage;    btn_right_top:            TImage;

{ * LCD SCREEN GRAPHICS (SPRITES)                                            * }

    // on-screen panel (alarm, time/score, game a/b, etc.)
    im_panel_game_a: TImage; im_panel_game_b: TImage;

    im_panel_time_1: TImage; im_panel_time_2: TImage; im_panel_alarm: TImage; im_panel_date: TImage; im_panel_chrono: TImage;

    im_panel_time_am: TImage; im_panel_time_pm: TImage; im_panel_time_colon: TImage;

    im_panel_sound: TImage; im_panel_bell: TImage;

    im_panel_digit_pos_1: TImage; im_panel_digit_pos_2: TImage; im_panel_digit_pos_3: TImage; im_panel_digit_pos_4: TImage; im_panel_digit_pos_5: TImage;

    im_panel_miss_1: TImage; im_panel_miss_2: TImage; im_panel_miss_3: TImage;

    im_panel_number_0: TImage; im_panel_number_1: TImage; im_panel_number_2: TImage; im_panel_number_3: TImage; im_panel_number_4: TImage;
    im_panel_number_5: TImage; im_panel_number_6: TImage; im_panel_number_7: TImage; im_panel_number_8: TImage; im_panel_number_9: TImage;

    im_panel_number_small_0: TImage; im_panel_number_small_1: TImage; im_panel_number_small_2: TImage; im_panel_number_small_3: TImage; im_panel_number_small_4: TImage;
    im_panel_number_small_5: TImage; im_panel_number_small_6: TImage; im_panel_number_small_7: TImage; im_panel_number_small_8: TImage; im_panel_number_small_9: TImage;

    // on-screen items
    im_hero_1_1: TImage; im_hero_1_2: TImage; im_hero_1_3: TImage; im_hero_1_4: TImage;
    im_hero_2_1: TImage; im_hero_2_2: TImage; im_hero_2_3: TImage; im_hero_2_4: TImage;
    im_hero_3_1: TImage; im_hero_3_2: TImage; im_hero_3_3: TImage; im_hero_3_4: TImage; im_hero_3_5: TImage; im_hero_3_6: TImage;

    im_item_1_1: TImage; im_item_1_2: TImage; im_item_1_3: TImage; im_item_1_4: TImage; im_item_1_5: TImage; im_item_1_6: TImage;
    im_item_2_1: TImage; im_item_2_2: TImage; im_item_2_3: TImage; im_item_2_4: TImage; im_item_2_5: TImage; im_item_2_6: TImage;
    im_item_3_1: TImage; im_item_3_2: TImage; im_item_3_3: TImage; im_item_3_4: TImage; im_item_3_5: TImage; im_item_3_6: TImage;

    im_item_1_got: TImage; im_item_2_got: TImage; im_item_3_got: TImage;

    im_item_1_miss: TImage; im_item_2_miss: TImage; im_item_3_miss: TImage;

    im_item_top_1_1: TImage; im_item_top_1_2: TImage; im_item_top_1_3: TImage;
    im_item_top_2_1: TImage; im_item_top_2_2: TImage; im_item_top_2_3: TImage; im_item_top_2_4: TImage; im_item_top_2_5: TImage;
    im_item_top_3_1: TImage; im_item_top_3_2: TImage; im_item_top_3_3: TImage; im_item_top_3_4: TImage;

{ * TIMERS                                                                   * }
    Timer_Bonus:      TTimer;
    Timer_Clock:      TTimer;
    Timer_Date:       TTimer;
    Timer_Game:       TTimer;
    Timer_Game_Over:  TTimer;
    Timer_Game_Start: TTimer;
    Timer_Got:        TTimer;
    Timer_Reprise:    TTimer;
    Timer_Startup:    TTimer;

{ * POPUP MENU                                                               * }
    pm_0_menu:          TPopupMenu;
    pm_1_about:         TMenuItem;
    pm_2_readme:        TMenuItem;
    pm_3_whatsnew:      TMenuItem;
    pm_4_manual:        TMenuItem;
    pm_5_n1:            TMenuItem;
    pm_6_config:        TMenuItem;
    pm_7_keys:          TMenuItem;
    pm_8_hiscore:       TMenuItem;
    pm_9_n2:            TMenuItem;
  {}pm_10_fullscreen:   TMenuItem;
  {}pm_11_splashscreen: TMenuItem;
{}//pm_12_controller:   TMenuItem; (not required)
  {}pm_13_n3:           TMenuItem;
  {}pm_14_exit:         TMenuItem;
    im_background: TImage;
    im_item_top_3_5: TImage;

{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
    procedure FormCreate                 (Sender: TObject);
    procedure FormActivate               (Sender: TObject);
    procedure FormClose                  (Sender: TObject; var Action: TCloseAction);

{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
    procedure Vid_Show_All               (b: boolean);

    procedure Vid_Show_All_Hero          (b: boolean);
    procedure Vid_Show_All_Item          (b: boolean);
    procedure Vid_Show_All_Item_Got      (b: boolean);
    procedure Vid_Show_All_Item_Miss     (b: boolean);
    procedure Vid_Show_All_Item_Top      (b: boolean);

    procedure Vid_Show_Hero              (var p: integer; q: integer);
    procedure Vid_Show_Item_Top          (var p: integer);

    procedure Vid_Show_Panel_Score       (var p: integer; b: boolean);
    procedure Vid_Show_Panel_Miss        (var p: integer);

{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
    procedure Snd_Play                   (pfs: PFSoundSample);
    procedure Snd_Silence;

{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
    procedure im_backgroundMouseDown     (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure im_backgroundMouseMove     (Sender: TObject;                       Shift: TShiftState; X, Y: Integer);

    procedure btn_game_a_topMouseDown    (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_a_topMouseUp      (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_b_topMouseDown    (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_b_topMouseUp      (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_time_topMouseDown      (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_time_topMouseUp        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure btn_left_topMouseDown      (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_left_topMouseUp        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_right_topMouseDown     (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_right_topMouseUp       (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure FormKeyDown                (Sender: TObject; var Key: Word;        Shift: TShiftState);
    procedure FormKeyUp                  (Sender: TObject; var Key: Word;        Shift: TShiftState);

{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
    procedure Inp_game_a_down;           procedure Inp_game_a_up;
    procedure Inp_game_b_down;           procedure Inp_game_b_up;
    procedure Inp_time_down;             procedure Inp_time_up;

    procedure Inp_left_down;             procedure Inp_left_up;
    procedure Inp_right_down;            procedure Inp_right_up;

{ * PROCEDURES: GAME ROUTINES                                                * }
    procedure Gam_Stop_All_Timers;

    procedure Gam_Set_Level;
    procedure Gam_Set_Mode;

{ * PROCEDURES: TIMERS                                                       * }
    procedure Timer_BonusTimer           (Sender: TObject);
    procedure Timer_ClockTimer           (Sender: TObject);
    procedure Timer_DateTimer            (Sender: TObject);
    procedure Timer_GameTimer            (Sender: TObject);
    procedure Timer_Game_OverTimer       (Sender: TObject);
    procedure Timer_Game_StartTimer      (Sender: TObject);
    procedure Timer_GotTimer             (Sender: TObject);
    procedure Timer_RepriseTimer         (Sender: TObject);
    procedure Timer_StartupTimer         (Sender: TObject);

{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
    procedure pm_0_menuPopup             (Sender: TObject);
    procedure pm_1_aboutClick            (Sender: TObject);
    procedure pm_2_readmeClick           (Sender: TObject);
    procedure pm_3_whatsnewClick         (Sender: TObject);
    procedure pm_4_manualClick           (Sender: TObject);
    procedure pm_6_configClick           (Sender: TObject);
    procedure pm_7_keysClick             (Sender: TObject);
    procedure pm_8_hiscoreClick          (Sender: TObject);
  {}procedure pm_10_fullscreenClick      (Sender: TObject);
  {}procedure pm_11_splashscreenClick    (Sender: TObject);
{}//procedure pm_12_controllerClick      (Sender: TObject);
  {}procedure pm_14_exitClick            (Sender: TObject);

{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
    procedure btn_game_menu_topClick     (Sender: TObject);

    procedure la_menu_itemMouseEnter     (Sender: TObject);
    procedure la_menu_itemMouseLeave     (Sender: TObject);
    procedure la_menu_itemMouseUp        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure Vid_Display_Game_Menu      (b: boolean);
  end;

const
{ introduced for use with HH2 / now incorporated into main source }

  HH2_CONTENT_NAME    = 'Asterix: Hunt for the Boars (Ludotronic)';
  HH2_CONTENT_AUTHOR  = 'Luca "MADrigal" Antignano';
  HH2_CONTENT_EMAIL   = 'lucantignano@gmail.com';
  HH2_CONTENT_VERSION = '5.00';
  HH2_CONTENT_DATE    = '2023/09/04';
  HH2_CONTENT_WEBSITE = 'www.madrigaldesign.it/sim';

  { constants for all builds }
{SK}INIFILE       = 'Config.ini';                                               {SK: now exclusively related to the executable configuration file pointing to the various skins}

  APP_NAME    = HH2_CONTENT_NAME + ' Simulator';                                // application's name (appears on taskbar and Splash screen)
{}APP_VERSION = 'Version ' + HH2_CONTENT_VERSION + ' * Multi-skin System, Final *'; // current version (appears on Splash screen)
{}APP_DATE    = 'Released by ' + HH2_CONTENT_AUTHOR + ' on 4th September, 2023';// release date (appears on Splash screen)

  URL_EMAIL   = 'mailto:' + HH2_CONTENT_EMAIL;                                  // link to the email address
  URL_PAYPAL  = 'https://www.paypal.me/LucaAntignano';                          // link to the Paypal donation page
  URL_WEBSITE = 'http://' + HH2_CONTENT_WEBSITE + '/';                          // link to the website

  REG_KEY     = 'Software\MADrigal\AsterixH50';                                 // registry key

  HINT_CLICK  = 'click or press';
  HINT_GAME_A = 'to start game A';
  HINT_GAME_B = 'to start game B';
  HINT_TIME   = 'to switch mode';
  HINT_SET    = 'button disabled';
  HINT_MOVE   = 'arrow keys';

{}MENU_ITEMS = 13;
  NUM_KEYS = 5;

var
{ * main objects * }
  Form1:                   TForm1;                                              // main form
  bStartup:                boolean;                                             // manages splash screen at startup
  iMouseX, iMouseY:        integer;                                             // cursor coordinates (to drag main form)
  iReg:                    TRegIniFile;                                         // system registry (to save high-scores)

{SK}{ * inifiles management * }
{SK}iNumSkins,                                                                  // number of skins present as read from the main 'config.ini' file
{SK}iSelectedSkin          : integer;                                           // index of selected skin (as result of combobox in Splash Screen
{SK}
{SK}aSkin                  : array[1..99] of record                             // how many skins present in the main 'config.ini' file
{SK}                                           Path,
{SK}                                           Name,
{SK}                                           SkinFile,
{SK}                                           SkinPreviewImg: string;
{SK}                                         end;

{ * sound management * }
  pfs_gameover,
  pfs_got,
  pfs_miss,
  pfs_move,
  pfs_start:               PFSoundSample;                                       // sound samples

  bSound,                                                                       // sound enabled? (depending on selected driver on splash screen)
  bSoundIngame:            boolean;                                             // sound enabled? (depending on selection when starting game)

{ * arrays of images * }
  aIm_panel_digit_pos:     array[1..5]       of TImage;                         // empty sprites to host score/time numbers
  aIm_panel_number,
  aIm_panel_number_small:  array[0..9]       of TImage;                         // numbers for score/time

  aIm_panel_miss:          array[1..3]       of TImage;                         // misses/chrono counter sprites

  aIm_hero,                                                                     // main character sprites       (4,4,6 sprites)
  aIm_item:                array[1..3, 1..6] of TImage;                         // moveable items               (6,6,6 sprites)
  aIm_item_miss,                                                                // missed items
  aIm_item_got:            array[1..3]       of TImage;                         // got items
  aIm_item_top:            array[1..3, 1..5] of TImage;                         // items on top of playground   (3,5,5 sprites)

{ * arrays of numbers *}
  aItems:                  array[1..3, 1..6] of integer;                        // 0=empty; 1=item moving; 2:item just got
  aItemsMiss:              array[1..3]       of integer;                        // 0=empty; 1=item missed
  aItemsGot:               array[1..3]       of integer;                        // 0=empty; 1=about to be removed

{ * arrays of mixed types * }
  aKeys:                   array[1..5]        of record                         // keys management (values)
                                                   int:  integer;
                                                   word: word;
                                                 end;

  aKeyHints:               array[1..44]       of string;                        // keys management (hint on buttons)

  aMenuItem:               array[1..MENU_ITEMS] of TMenuItem;                   // popup menu items
  aMenuLabels:             array[1..MENU_ITEMS] of TLabel;                      // on-screen game menu labels

{ * integers and booleans * }
  iMode,                                                                        // 0=ACL, 1=time, etc.
  iScore,                                                                       // 0-9999
  iHiScoreA,                                                                    // 0-9999
  iHiScoreB,                                                                    // 0-9999
  iMiss,                                                                        // 0-5
  iHeroPos                                                                      // 1-3
                         : integer;

{}bFullscreenMode,                                                              // true if video mode is fullscreen (false if "shaped game")
  bGameMenuOpen,                                                                // true if the on-screen menu is visible

  bKeyPressed,                                                                  // true while left/right arrow keys are being kept pressed
  bGameAPressed,                                                                // true while 'Game A' button is being kept pressed
  bGameBPressed,                                                                // true while 'Game B' button is being kept pressed
  bControllers                                                                  // true while player can actually operate the main character
                         : boolean;

  iLoop,                                                                        // 1-3 for "TimerGame"
  iLoopTop,                                                                     // 1-4 for "TimerGame" (top center item animation)
  iLoopGot,                                                                     // 1-2 used in TimerGot
  iLoopBonus,                                                                   // 1-? used in TimerBonus
  iLoopMiss,                                                                    // 1-? used in TimerReprise and TimerGameOver
  iMaxOnscreen                                                                  // 1-? (maximum allowed moveable items on screen simultaneously)
                         : integer;

implementation

uses
  about, gfxinit, keys, manual, splash;

{$r *.dfm}










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.FormCreate(Sender: TObject);
{ sprites on form are associated to arrays of images, needed to "index" them   }
{ variables are initialized: system registry, randomize, game mode, etc.       }
  var
    i: integer;
{SK}bError: boolean;
begin
  // initializes application name (for Windows taskbar)
  Application.Title := APP_NAME;

{}// sets game off the screen while graphics initialise
{}Top  := - Screen.Height;
{}Left := - Screen.Width;

{SK}// skin configuration files management
{SK}bError := false;

{SK}if not FileExists(ExtractFilePath(ParamStr(0)) + INIFILE) then
{SK}  bError := true
{SK}else
{SK}  begin
{SK}    IF_File := TIniFile.Create(ExtractFilePath(ParamStr(0)) + INIFILE);
{SK}
{SK}    iNumSkins := IF_file.ReadInteger('Common', 'Skins', 2);
{SK}
{SK}    for i := 1 to iNumSkins do
{SK}      begin
{SK}        aSkin[i].Path := IncludeTrailingPathDelimiter(IF_file.ReadString('Skin' + IntToStr(i), 'Path', 'Graphics\Skin' + IntToStr(i) + '\'));
{SK}        aSkin[i].Name := IF_file.ReadString('Skin' + IntToStr(i), 'Name', 'Nameless Skin');
{SK}        aSkin[i].SkinFile := IF_file.ReadString('Skin' + IntToStr(i), 'SkinFile', 'Skin.ini');
{SK}        aSkin[i].SkinPreviewImg := IF_file.ReadString('Skin' + IntToStr(i), 'SkinPreviewImg', 'SkinPreview.png');
{SK}      end;
{SK}    IF_File.Destroy;
{SK}  end;
{SK}
{SK}if bError then
{SK}  begin
{SK}    MessageDlg('Configuration file (' + INIFILE + ') not found. Please reinstall the game.', mtError, [mbOK], 0);
{SK}    Application.Terminate;
{SK}  end;

  // on-screen panel (alarm, time/score, game a/b, etc.)
  aIm_panel_digit_pos[1] := im_panel_digit_pos_1; aIm_panel_digit_pos[2] := im_panel_digit_pos_2; aIm_panel_digit_pos[3] := im_panel_digit_pos_3; aIm_panel_digit_pos[4] := im_panel_digit_pos_4; aIm_panel_digit_pos[5] := im_panel_digit_pos_5;

  aIm_panel_number[0] := im_panel_number_0; aIm_panel_number[1] := im_panel_number_1; aIm_panel_number[2] := im_panel_number_2; aIm_panel_number[3] := im_panel_number_3; aIm_panel_number[4] := im_panel_number_4;
  aIm_panel_number[5] := im_panel_number_5; aIm_panel_number[6] := im_panel_number_6; aIm_panel_number[7] := im_panel_number_7; aIm_panel_number[8] := im_panel_number_8; aIm_panel_number[9] := im_panel_number_9;

  aIm_panel_number_small[0] := im_panel_number_small_0; aIm_panel_number_small[1] := im_panel_number_small_1; aIm_panel_number_small[2] := im_panel_number_small_2; aIm_panel_number_small[3] := im_panel_number_small_3; aIm_panel_number_small[4] := im_panel_number_small_4;
  aIm_panel_number_small[5] := im_panel_number_small_5; aIm_panel_number_small[6] := im_panel_number_small_6; aIm_panel_number_small[7] := im_panel_number_small_7; aIm_panel_number_small[8] := im_panel_number_small_8; aIm_panel_number_small[9] := im_panel_number_small_9;

  aIm_panel_miss[1] := im_panel_miss_1; aIm_panel_miss[2] := im_panel_miss_2; aIm_panel_miss[3] := im_panel_miss_3;

  // on-screen items
  aIm_hero[1, 1] := im_hero_1_1; aIm_hero[1, 2] := im_hero_1_2; aIm_hero[1, 3] := im_hero_1_3; aIm_hero[1, 4] := im_hero_1_4;
  aIm_hero[2, 1] := im_hero_2_1; aIm_hero[2, 2] := im_hero_2_2; aIm_hero[2, 3] := im_hero_2_3; aIm_hero[2, 4] := im_hero_2_4;
  aIm_hero[3, 1] := im_hero_3_1; aIm_hero[3, 2] := im_hero_3_2; aIm_hero[3, 3] := im_hero_3_3; aIm_hero[3, 4] := im_hero_3_4; aIm_hero[3, 5] := im_hero_3_5; aIm_hero[3, 6] := im_hero_3_6;

  aIm_item[1, 1] := im_item_1_1; aIm_item[1, 2] := im_item_1_2; aIm_item[1, 3] := im_item_1_3; aIm_item[1, 4] := im_item_1_4; aIm_item[1, 5] := im_item_1_5; aIm_item[1, 6] := im_item_1_6;
  aIm_item[2, 1] := im_item_2_1; aIm_item[2, 2] := im_item_2_2; aIm_item[2, 3] := im_item_2_3; aIm_item[2, 4] := im_item_2_4; aIm_item[2, 5] := im_item_2_5; aIm_item[2, 6] := im_item_2_6;
  aIm_item[3, 1] := im_item_3_1; aIm_item[3, 2] := im_item_3_2; aIm_item[3, 3] := im_item_3_3; aIm_item[3, 4] := im_item_3_4; aIm_item[3, 5] := im_item_3_5; aIm_item[3, 6] := im_item_3_6;

  aIm_item_miss[1] := im_item_1_miss; aIm_item_miss[2] := im_item_2_miss; aIm_item_miss[3] := im_item_3_miss;

  aIm_item_got[1] := im_item_1_got; aIm_item_got[2] := im_item_2_got; aIm_item_got[3] := im_item_3_got;

  aIm_item_top[1, 1] := im_item_top_1_1; aIm_item_top[1, 2] := im_item_top_1_2; aIm_item_top[1, 3] := im_item_top_1_3;
  aIm_item_top[2, 1] := im_item_top_2_1; aIm_item_top[2, 2] := im_item_top_2_2; aIm_item_top[2, 3] := im_item_top_2_3; aIm_item_top[2, 4] := im_item_top_2_4; aIm_item_top[2, 5] := im_item_top_2_5;
  aIm_item_top[3, 1] := im_item_top_3_1; aIm_item_top[3, 2] := im_item_top_3_2; aIm_item_top[3, 3] := im_item_top_3_3; aIm_item_top[3, 4] := im_item_top_3_4; aIm_item_top[3, 5] := im_item_top_3_5;

  // registry keys management
  iReg := TregIniFile.Create(REG_KEY);                                          // system registry init

{}// init graphics
{}bFullscreenMode := iReg.ReadBool('', 'FullscreenMode', true);
{}pm_10_fullscreen.Checked := bFullscreenMode;
{SK: clause "Gfx_Game_Init;" moved into the 'Button Play Click' procedure in splash.pas
{}bGameMenuOpen := false;

{}// initialises the "hints" array
  for i := 1 to 10 do
    aKeyHints[i] := '''' + Chr(i + 47) + '''';

  for i := 11 to 36 do
    aKeyHints[i] := '''' + Chr(i + 54) + '''';

  aKeyHints[37] := '''up arrow'' key';
  aKeyHints[38] := '''down arrow'' key';
  aKeyHints[39] := '''left arrow'' key';
  aKeyHints[40] := '''right arrow'' key';
  aKeyHints[41] := '''alt'' key';
  aKeyHints[42] := '''ctrl'' key';
  aKeyHints[43] := '''shift'' key';
  aKeyHints[44] := '''spacebar''';

  // reads values from system registry into the inputs array
  aKeys[1].int := iReg.ReadInteger('', 'Key1', 2);                              // "Game A" (default: "1")
  aKeys[2].int := iReg.ReadInteger('', 'Key2', 3);                              // "Game B" (default: "2")
  aKeys[3].int := iReg.ReadInteger('', 'Key3', 4);                              // "Time"   (default: "3")
  aKeys[4].int := iReg.ReadInteger('', 'Key4', 39);                             // "Left"   (default: "LEFT")
  aKeys[5].int := iReg.ReadInteger('', 'Key5', 40);                             // "Right"  (default: "RIGHT")

  // writes values into the system registry (in case no pre-existing values
  // stored), then assigns key values to the key-inputs array
  for i := 1 to NUM_KEYS do
    begin
      iReg.WriteInteger('', 'Key' + inttostr(i), aKeys[i].int);
      KeysForm.AssignWord(i, aKeys[i].int);
    end;

  // sets button images hints
  btn_game_a_top.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[1].int] + ' ' + HINT_GAME_A; // "Game A"
  btn_game_b_top.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[2].int] + ' ' + HINT_GAME_B; // "Game B"
  btn_time_top.Hint   := HINT_CLICK + ' ' + aKeyHints[aKeys[3].int] + ' ' + HINT_TIME;   // "Time"
  btn_set_top.Hint    := HINT_SET;                                                       // "Set"

  btn_left_top.Hint   := HINT_CLICK + ' ' + aKeyHints[aKeys[4].int];                     // "Left"
  btn_right_top.Hint  := HINT_CLICK + ' ' + aKeyHints[aKeys[5].int];                     // "Right"

  // popup menu array association
  aMenuItem[1]  := pm_1_about;
  aMenuItem[2]  := pm_2_readme;
  aMenuItem[3]  := pm_3_whatsnew;
  aMenuItem[4]  := pm_4_manual;
  aMenuItem[5]  := pm_5_n1;
  aMenuItem[6]  := pm_6_config;
  aMenuItem[7]  := pm_7_keys;
  aMenuItem[8]  := pm_8_hiscore;
  aMenuItem[9]  := pm_9_n2;
{}aMenuItem[10] := pm_10_fullscreen;
{}aMenuItem[11] := pm_11_splashscreen;
{}//               pm_12_controller;
{}aMenuItem[12] := pm_13_n3;
{}aMenuItem[13] := pm_14_exit;

  // variables init
  DoubleBuffered := true;                                                       // enables double buffering for rendering of "form1"

  iHiScoreA := iReg.ReadInteger('', 'HiScore1', iHiScoreA);                     // reads value from system registry into iHiScoreA
  iHiScoreB := iReg.ReadInteger('', 'HiScore2', iHiScoreB);                     // reads value from system registry into iHiScoreB

  bKeyPressed := false;                                                         // no key is currently being kept pressed
  bGameAPressed := false;                                                       // 'Game A' button is not currently pressed
  bGameBPressed := false;                                                       // 'Game B' button is not currently pressed
  bControllers := false;                                                        // prevents main character movement

  Randomize;                                                                    // initializes the random numbers generator
end;





procedure TForm1.FormActivate(Sender: TObject);
{ sends focus to the splash screen at startup                                  }
begin
  if bStartup then
    begin
      bStartup := false;
      SplashForm.Show;
    end;
end;





procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
{ frees memory before shutting down                                            }
begin
  FSOUND_Sample_Free(pfs_gameover); FSOUND_Sample_Free(pfs_got);
  FSOUND_Sample_Free(pfs_miss);     FSOUND_Sample_Free(pfs_move);
  FSOUND_Sample_Free(pfs_start);

  FSOUND_Close;

{}iReg.Destroy;
  Action := caFree;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Vid_Show_All(b: boolean);
{ displays/hides nearly all sprites (except score panel)                       }
  var
    i: integer;
begin
  im_panel_game_a.Visible     := b;
  im_panel_game_b.Visible     := b;
  im_panel_time_1.Visible     := b;
  im_panel_time_2.Visible     := b;
  im_panel_alarm.Visible      := b;
  im_panel_date.Visible       := b;
  im_panel_chrono.Visible     := b;
  im_panel_bell.Visible       := b;
  im_panel_sound.Visible      := b;
  im_panel_time_am.Visible    := b;
  im_panel_time_pm.Visible    := b;
  im_panel_time_colon.Visible := b;

  for i := 1 to 5 do
    aIm_panel_digit_pos[i].Visible := true;

  i := 8888; Vid_Show_Panel_Score(i, b);

  if b then
    aIm_panel_digit_pos[5].Picture := aIm_panel_number_small[8].Picture
  else
    aIm_panel_digit_pos[5].Picture := nil;

  i := Ord(b) * 3; Vid_Show_Panel_Miss(i);

  Vid_Show_All_Hero(b);
  Vid_Show_All_Item(b);
  Vid_Show_All_Item_Miss(b);
  Vid_Show_All_Item_Got(b);
  Vid_Show_All_Item_Top(b);
end;





procedure TForm1.Vid_Show_All_Hero(b: boolean);
{ displays/hides all main character sprites                                    }
  var
    i, j: integer;
begin
  for i := 1 to 3 do
    for j := 1 to 4 do
      aIm_hero[i, j].Visible := b;

  for j := 5 to 6 do
    aIm_hero[3, j].Visible := b;
end;





procedure TForm1.Vid_Show_All_Item(b: boolean);
{ displays/hides all 'moveable' items                                          }
  var
    i, j: integer;
begin
  for i := 1 to 3 do
    for j := 1 to 6 do
      aIm_item[i, j].Visible := b;
end;





procedure TForm1.Vid_Show_All_Item_Got(b: boolean);
{ displays/hides all 'got' items                                               }
  var
    i: integer;
begin
  for i := 1 to 3 do
    aIm_item_got[i].Visible := b;
end;





procedure TForm1.Vid_Show_All_Item_Miss(b: boolean);
{ displays/hides all 'missed' items                                            }
  var
    i: integer;
begin
  for i := 1 to 3 do
    aIm_item_miss[i].Visible := b;
end;





procedure TForm1.Vid_Show_All_Item_Top(b: boolean);
{ displays/hides all 'got' items                                               }
  var
    i, j: integer;
begin
  for j := 1 to 3 do
    aIm_item_top[1, j].Visible := b;

  for i := 2 to 3 do
    for j := 1 to 5 do
      aIm_item_top[i, j].Visible := b;
end;





procedure TForm1.Vid_Show_Hero(var p: integer; q: integer);
{ shows/hides main character in 'p' position, in 'q' status                    }
  var
    i: integer;
begin
  case q of
    0: for i := 1 to 3 do                                                       // all char sprites hidden
         aIm_hero[p, i].Visible := false;
    1: begin                                                                    // char sprites (p, 1) and (p, 2) visible
         aIm_hero[p, 1].Visible := true;
         aIm_hero[p, 2].Visible := true;
         aIm_hero[p, 3].Visible := false;
       end;
    2: begin                                                                    // char sprites (p, 1) and (p, 3) visible
         aIm_hero[p, 1].Visible := true;
         aIm_hero[p, 2].Visible := false;
         aIm_hero[p, 3].Visible := true;
       end;
  end; // case..of
end;





procedure TForm1.Vid_Show_Item_Top(var p: integer);
{ shows/hides the character at the top of screen in 'p' status                 }
  var
    i, j: integer;
begin
  case p of
    0: Vid_Show_All_Item_Top(false);                                            // hides all items
    1: begin                                                                    // item_top 1,1 and 1,2
         aIm_item_top[1, 1].Visible := true;
         aIm_item_top[1, 2].Visible := true;
         aIm_item_top[1, 3].Visible := false;

         for i := 2 to 3 do
           for j := 1 to 5 do
             aIm_item_top[i, j].Visible := false;
       end;
    2: begin                                                                    // item_top 1,1 and 1,3
         aIm_item_top[1, 1].Visible := true;
         aIm_item_top[1, 2].Visible := false;
         aIm_item_top[1, 3].Visible := true;

         for i := 2 to 3 do
           for j := 1 to 5 do
             aIm_item_top[i, j].Visible := false;
       end;
    3: begin                                                                    // item_top 2,1 and 2,2
         for j := 1 to 3 do
           aIm_item_top[1, j].Visible := false;

         aIm_item_top[2, 1].Visible := true;
         aIm_item_top[2, 2].Visible := true;
         aIm_item_top[2, 3].Visible := false;
         aIm_item_top[2, 4].Visible := false;
         aIm_item_top[2, 5].Visible := false;

         for j := 1 to 5 do
           aIm_item_top[3, j].Visible := false;
       end;
    4: begin                                                                    // item_top 2,1 and 2,3
         for j := 1 to 3 do
           aIm_item_top[1, j].Visible := false;

         aIm_item_top[2, 1].Visible := true;
         aIm_item_top[2, 2].Visible := false;
         aIm_item_top[2, 3].Visible := true;
         aIm_item_top[2, 4].Visible := false;
         aIm_item_top[2, 5].Visible := false;

         for j := 1 to 5 do
           aIm_item_top[3, j].Visible := false;
       end;
    5: begin                                                                    // item_top 2,1 and 2,4
         for j := 1 to 3 do
           aIm_item_top[1, j].Visible := false;

         aIm_item_top[2, 1].Visible := true;
         aIm_item_top[2, 2].Visible := false;
         aIm_item_top[2, 3].Visible := false;
         aIm_item_top[2, 4].Visible := true;
         aIm_item_top[2, 5].Visible := false;

         for j := 1 to 5 do
           aIm_item_top[3, j].Visible := false;
       end;
    6: begin                                                                    // item_top 2,1 and 2,5
         for j := 1 to 3 do
           aIm_item_top[1, j].Visible := false;

         aIm_item_top[2, 1].Visible := true;
         aIm_item_top[2, 2].Visible := false;
         aIm_item_top[2, 3].Visible := false;
         aIm_item_top[2, 4].Visible := false;
         aIm_item_top[2, 5].Visible := true;

         for j := 1 to 5 do
           aIm_item_top[3, j].Visible := false;
       end;
    7: begin                                                                    // item_top 3,1
         for j := 1 to 3 do
           aIm_item_top[1, j].Visible := false;

         for j := 1 to 5 do
           aIm_item_top[2, j].Visible := false;

         aIm_item_top[3, 1].Visible := true;

         for j := 2 to 5 do
           aIm_item_top[3, j].Visible := false;
       end;
    8: begin                                                                    // item_top 3,2 and 3,3
         for j := 1 to 3 do
           aIm_item_top[1, j].Visible := false;

         for j := 1 to 5 do
           aIm_item_top[2, j].Visible := false;

         aIm_item_top[3, 1].Visible := false;
         aIm_item_top[3, 2].Visible := true;
         aIm_item_top[3, 3].Visible := true;
         aIm_item_top[3, 4].Visible := false;
         aIm_item_top[3, 5].Visible := false;
       end;
    9: begin                                                                    // item_top 3,2 and 3,4
         for j := 1 to 3 do
           aIm_item_top[1, j].Visible := false;

         for j := 1 to 5 do
           aIm_item_top[2, j].Visible := false;

         aIm_item_top[3, 1].Visible := false;
         aIm_item_top[3, 2].Visible := true;
         aIm_item_top[3, 3].Visible := false;
         aIm_item_top[3, 4].Visible := true;
         aIm_item_top[3, 5].Visible := false;
       end;
   10: begin                                                                    // item_top 3,2 and 3,5
         for j := 1 to 3 do
           aIm_item_top[1, j].Visible := false;

         for j := 1 to 5 do
           aIm_item_top[2, j].Visible := false;

         aIm_item_top[3, 1].Visible := false;
         aIm_item_top[3, 2].Visible := true;
         aIm_item_top[3, 3].Visible := false;
         aIm_item_top[3, 4].Visible := false;
         aIm_item_top[3, 5].Visible := true;
       end;
  end; // case..of
end;





procedure TForm1.Vid_Show_Panel_Score(var p: integer; b: boolean);
{ transforms score/high-score into sprites, and displays/hides them            }
  var
    i, t: integer;
begin
  t := p;

  while t >= 10000 do                                                           // displays "10000" as "...0"
    Dec(t, 10000);

  if b then                                                                     // display or hide score/high-score?
    begin                                                                               // display
      if t > 999 then aIm_panel_digit_pos[1].Picture := aIm_panel_number[t div 1000 mod 10].Picture else aIm_panel_digit_pos[1].Picture := nil; // thousands
      if t > 99  then aIm_panel_digit_pos[2].Picture := aIm_panel_number[t div  100 mod 10].Picture else aIm_panel_digit_pos[2].Picture := nil; // hundreds
      if t > 9   then aIm_panel_digit_pos[3].Picture := aIm_panel_number[t div   10 mod 10].Picture else aIm_panel_digit_pos[3].Picture := nil; // tens
                      aIm_panel_digit_pos[4].Picture := aIm_panel_number[t          mod 10].Picture;                                            // units
                      aIm_panel_digit_pos[5].Picture := nil;
    end
  else                                                                                  // hide
    for i := 1 to 5 do
      aIm_panel_digit_pos[i].Picture := nil;
end;





procedure TForm1.Vid_Show_Panel_Miss(var p: integer);
{ displays as many sprites as p on the misses counter (0=nothing visible)      }
  var
    i: integer;
begin
  for i := 1 to 3 do
    aIm_panel_miss[i].Visible := (i <= p);                                      // displays items 1 to p, hides the remaining ones
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Snd_Play(pfs: PFSoundSample);
{ plays a certain sound sample                                                 }
begin
  if bSound and bSoundIngame then
    FSOUND_PlaySound(FSOUND_FREE, pfs);
end;





procedure TForm1.Snd_Silence;
{ stops any currently playing sound sample                                     }
begin
  if bSound and bSoundIngame then
    FSOUND_StopSound(FSOUND_ALL);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.im_backgroundMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: PRESS                                                }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (Button = mbLeft) then
    begin
      iMouseX := X;
      iMouseY := Y;
    end;
end;





procedure TForm1.im_backgroundMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: MOVE                                                 }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (ssLeft in Shift) then
    begin
      if Top + Y - iMouseY < 0 then
        Top := Top + Y - iMouseY
      else
        iMouseY := Y;

      if Top < -ClientHeight + 1 then
        Top := -ClientHeight + 1;

      Left := Left + X - iMouseX;
    end;
end;





procedure TForm1.btn_game_a_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ GAME A button: PRESS                                                         }
begin
  if Button = mbLeft then
    Inp_game_a_down;
end;





procedure TForm1.btn_game_a_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ GAME A button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_game_a_up;
end;





procedure TForm1.btn_game_b_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y:Integer);
{ GAME B button: PRESS                                                         }
begin
  if Button = mbLeft then
    Inp_game_b_down;
end;





procedure TForm1.btn_game_b_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ GAME B button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_game_b_up;
end;





procedure TForm1.btn_time_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ TIME button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_time_down;
end;





procedure TForm1.btn_time_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ TIME button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_time_up;
end;





procedure TForm1.btn_left_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ LEFT button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_left_down;
end;





procedure TForm1.btn_left_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ LEFT button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_left_up;
end;





procedure TForm1.btn_right_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ RIGHT button: PRESS                                                          }
begin
  if Button = mbLeft then
    Inp_right_down;
end;





procedure TForm1.btn_right_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ RIGHT button: RELEASE                                                        }
begin
  if Button = mbLeft then
    Inp_right_up;
end;





procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: PRESS                                                              }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_game_a_down else                              // Game A
  if Key = aKeys[2].word then Inp_game_b_down else                              // Game B
  if Key = aKeys[3].word then Inp_time_down   else                              // Time
  if Key = aKeys[4].word then Inp_left_down   else                              // Left
  if Key = aKeys[5].word then Inp_right_down;                                   // Right
end;





procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: RELEASE                                                            }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_game_a_up else                                // Game A
  if Key = aKeys[2].word then Inp_game_b_up else                                // Game B
  if Key = aKeys[3].word then Inp_time_up   else                                // Time
  if Key = aKeys[4].word then Inp_left_up   else                                // Left
  if Key = aKeys[5].word then Inp_right_up;                                     // Right
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Inp_game_a_down;
{ GAME A button: PRESS                                                         }
begin
  btn_game_a_down.Visible := true;

  if not bGameAPressed then
    begin
      bGameAPressed := true;

      if (iMode = 0) then                                                       // ACL ->
        begin
          iMode := 1;                                                           // -> Time 1
          Gam_Set_Mode;
        end
      else
        if ( (iMode > 0) and (iMode < 6) ) or (iMode = 10) then                 // Time1, Time 2, Alarm, Chrono, Date, Game Over ->
          begin
            iMode := 6;                                                         // -> HiScore A
            Gam_Set_Mode;
          end;                                     
    end;
end;





procedure TForm1.Inp_game_a_up;
{ GAME A button: RELEASE                                                       }
begin
  btn_game_a_down.Visible := false;
  bGameAPressed := false;

  if (iMode = 6) then                                                           // HiScore A ->
    begin
      iMode := 8;                                                               // -> Game A
      Gam_Set_Mode;
    end;
end;





procedure TForm1.Inp_game_b_down;
{ GAME B button: PRESS                                                         }
begin
  btn_game_b_down.Visible := true;

  if not bGameBPressed then
    begin
      bGameBPressed := true;

      if (iMode = 0) then                                                       // ACL
        begin
          iMode := 1;                                                           // -> Time 1
          Gam_Set_Mode;
        end                                       
      else
        if ( (iMode > 0) and (iMode < 6) ) or (iMode = 10) then                 // Time1, Time 2, Alarm, Chrono, Date, Game Over ->
          begin
            iMode := 7;                                                         // ->  HiScore B
            Gam_Set_Mode;
          end;
    end;
end;





procedure TForm1.Inp_game_b_up;
{ GAME B button: RELEASE                                                       }
begin
  btn_game_b_down.Visible := false;
  bGameBPressed := false;

  if (iMode = 7) then                                                           // HiScore B ->
    begin
      iMode := 9;                                                               // -> Game B
      Gam_Set_Mode;
    end;
end;





procedure TForm1.Inp_time_down;
{ TIME button: PRESS                                                           }
begin
  btn_time_down.Visible := true;

  if not bKeyPressed then
    begin
      bKeyPressed := true;

      if ( (iMode >= 0) and (iMode < 6) ) or (iMode = 10) then                  // any mode except hiscore/game modes ->
        begin
          Inc(iMode);                                                           // -> Time 1 -> Time 2 -> Alarm -> Date -> Chrono ->

          if iMode > 5 then
            iMode := 1;

          Gam_Set_Mode;
        end;
    end;
end;





procedure TForm1.Inp_time_up;
{ TIME button: RELEASE                                                         }
begin
  btn_time_down.Visible := false;
  bKeyPressed := false;
end;





procedure TForm1.Inp_left_down;
{ LEFT button: PRESS                                                           }
begin
  btn_left_down.Visible := true;

  if not bKeyPressed then
    begin
      bKeyPressed := true;

      if (iMode = 6) or (iMode = 7) then                                        // HiScore A or HiScore B modes
        begin
          im_panel_sound.Visible := false;                                      // hides "sound" sprite
          bSoundIngame := false;                                                // disables sound

          Timer_Game_Start.Interval := 100;                                     // game begins immediately (no starting music played)
        end;

      if bControllers and (iHeroPos > 1) then
        begin
          Vid_Show_Hero(iHeroPos, 0);                                           // hides main character
          Dec(iHeroPos);                                                        // updates coordinate
          Vid_Show_Hero(iHeroPos, 1);                                           // displays main character
        end;
    end;
end;





procedure TForm1.Inp_left_up;
{ LEFT button: RELEASE                                                         }
begin
  btn_left_down.Visible := false;
  bKeyPressed := false;
end;





procedure TForm1.Inp_right_down;
{ RIGHT button: PRESS                                                          }
begin
  btn_right_down.Visible := true;

  if not bKeyPressed then
    begin
      bKeyPressed := true;

      if bControllers and (iHeroPos < 3)then
        begin
          Vid_Show_Hero(iHeroPos, 0);                                           // hides main character
          Inc(iHeroPos);                                                        // updates coordinates
          Vid_Show_Hero(iHeroPos, 1);                                           // displays main character
        end;
    end;
end;





procedure TForm1.Inp_right_up;
{ RIGHT button: RELEASE                                                        }
begin
  btn_right_down.Visible := false;
  bKeyPressed := false;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME ROUTINES                                                * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Gam_Stop_All_Timers;
{ stops all timers at once                                                     }
begin
  Timer_Bonus.Enabled      := false;
  Timer_Clock.Enabled      := false;
  Timer_Date.Enabled       := false;
  Timer_Game.Enabled       := false;
  Timer_Game_Over.Enabled  := false;
  Timer_Game_Start.Enabled := false;
  Timer_Got.Enabled        := false;
  Timer_Reprise.Enabled    := false;
  Timer_Startup.Enabled    := false;
end;





procedure TForm1.Gam_Set_Level;
{ sets number of maximum moveable items on screen simultaneously, and value    }
{ for the TimerGame interval, making game faster/slower depending on which     }
{ game is played (A or B) and current score                                    }
  var
    i: integer;
begin
  i := iScore;

  while i > 999 do
    Dec(i, 1000);

  case iMode of
    8: begin                                                                    // Game A
         // maximum moveable items on screen
         case i of
           0..30:    iMaxOnscreen := 1;                                         //   0....30
           40..490:  iMaxOnscreen := 2;                                         //  40...490
           500..990: iMaxOnscreen := 5;                                         // 500...990
         end; // case..of

         // game speed
         case i of
           0..90:    Timer_Game.Interval := 310;                                //   0...90
           100..190: Timer_Game.Interval := 285;                                // 100..190
           200..290: Timer_Game.Interval := 270;                                // 200..290
           300..590: Timer_Game.Interval := 255;                                // 300..590
           600..990: Timer_Game.Interval := 215;                                // 600..990
         end; // case..of
       end;
    9: begin                                                                    // Game B
         // maximum moveable items on screen
         case i of
           0..30:    iMaxOnscreen := 1;                                         //   0....30
           40..490:  iMaxOnscreen := 3;                                         //  40...490
           500..990: iMaxOnscreen := 5;                                         // 500...990
         end; // case..of

         case i of
           0..90:    Timer_Game.Interval := 285;                                //   0...90
           100..290: Timer_Game.Interval := 250;                                // 100..290
           300..490: Timer_Game.Interval := 225;                                // 300..490
           500..690: Timer_Game.Interval := 210;                                // 500..690
           700..990: Timer_Game.Interval := 185;                                // 700..990
         end; // case..of
       end;
  end; // case..of
end;





procedure TForm1.Gam_Set_Mode;
{ sets game/time/alarm/etc mode, depending on the value of the iMode variable  }
  var
    i, j: integer;
begin
  case iMode of
    0: begin                                                                    // ACL
         Vid_Show_All(true);                                                            // game is turned on

         Timer_Startup.Interval := 30000;
         Timer_Startup.Enabled := true;
       end;
    1: begin                                                                    // TIME 1
         Gam_Stop_All_Timers;                                                           // stops all timers
         Vid_Show_All(false);                                                           // hides all items, chars, etc.
         im_panel_time_1.Visible := true;                                               // displays "time 1"
         im_panel_time_colon.Visible := true;                                           // displays colon

         Timer_Clock.Interval := 1;
         Timer_Clock.Enabled := true;                                                   // clock starts ticking...
       end;
    2: begin                                                                    // TIME 2
         im_panel_time_1.Visible := false;                                              // hides "time 1"
         im_panel_time_2.Visible := true;                                               // displays "time 2"
       end;
    3: begin                                                                    // ALARM
         Gam_Stop_All_Timers;                                                           // stops all timers
         Vid_Show_All(false);                                                           // hides all items, chars, etc.
         im_panel_alarm.Visible := true;                                                // displays "alarm"
         im_panel_time_am.Visible := true;                                              // displays "PM"
         im_panel_time_colon.visible := true;                                           // displays colon
         i := 1200; Vid_Show_Panel_Score(i, true);                                      // 12:00
       end;
    4: begin                                                                    // DATE
         Gam_Stop_All_Timers;                                                           // stops all timers
         Vid_Show_All(false);                                                           // hides all items, chars, etc.
         im_panel_date.Visible := true;                                                 // displays "date"
         im_panel_time_colon.visible := true;                                           // displays colon

         Timer_Date.Interval := 1;
         Timer_Date.Enabled := true;                                                    // clock starts ticking...
       end;
    5: begin                                                                    // CHRONO
         Gam_Stop_All_Timers;                                                           // stops all timers
         Vid_Show_All(false);                                                           // hides all items, chars, etc.
         im_panel_chrono.Visible := true;                                               // displays "chrono"
         im_panel_time_colon.Visible := true;                                           // displays colon

         for i := 1 to 4 do
           aIm_panel_digit_pos[i].Picture := aIm_panel_number[0].Picture;               // 00:00 x

         aIm_panel_digit_pos[5].Picture := aIm_panel_number_small[0].Picture;           // xx:xx 0
       end;
    6, 7: begin                                                                 // HISCORE A / B
         Gam_Stop_All_Timers;
         Vid_Show_All(false);

         case iMode of
           6: begin im_panel_game_a.Visible := true; Vid_Show_Panel_Score(iHiScoreA, true); end;
           7: begin im_panel_game_b.Visible := true; Vid_Show_Panel_Score(iHiScoreB, true); end;
         end; // case..of

         bSoundIngame := true; im_panel_sound.Visible := true;                          // ingame sound is enabled
         iHeroPos := 2; Vid_Show_Hero(iHeroPos, 1);                                     // displays main character in x=2
         iLoopTop := 5; Vid_Show_Item_Top(iLoopTop);                                    // display items on top of screen

         Timer_Game_Start.Interval := 3100;                                             // initializes starting tune length (can be zero'ed by pressing LEFT button when in this mode)
       end;
    8, 9: begin                                                                 // GAME A / B
         iScore := 0; { Vid_Show_Panel_Score(iScore, true); }                           // initializes score (=0), displays it later
         iMiss := 0; Vid_Show_Panel_Miss(iMiss);                                        // display misses (=0)
         Gam_Set_Level;                                                                 // sets game speed, difficulty etc.

         for i := 1 to 3 do                                                             // clears moveable items array
           begin
             for j := 1 to 6 do
               aItems[i, j] := 0;

             aItemsGot[i] := 0;                                                         // clears "got" items array
             aItemsMiss[i] := 0;                                                        // clears "missed" items array
           end;

         Snd_Play(pfs_start);                                                           // plays "Start" tune

         Timer_Game_Start.Enabled := true;                                              // enables "Game Start" timer, then game starts
       end;
   10: begin                                                                    // GAME OVER
         // does nothing...
       end;
  end; // case..of
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: TIMERS                                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Timer_BonusTimer(Sender: TObject);
{ quick rest for a drink                                                       }
  var
    i, j: integer;
begin
  case iLoopBonus of
    1: begin Snd_Play(pfs_move); Vid_Show_Hero(iHeroPos, 0); Inc(iHeroPos); Vid_Show_Hero(iHeroPos, 1); end; // moves to x=2
    2: begin Snd_Play(pfs_move); Vid_Show_Hero(iHeroPos, 0); Inc(iHeroPos); Vid_Show_Hero(iHeroPos, 1); end; // moves to x=3
    3: begin Snd_Play(pfs_move); aIm_hero[3, 2].Visible := false; aIm_hero[3, 3].Visible := false; aIm_hero[3, 5].Visible := true; Timer_Bonus.Interval := 500; end; // gets ready to drink
    4: begin Snd_Play(pfs_move); aIm_hero[3, 5].Visible := false; aIm_hero[3, 6].Visible := true; end; // drinks
    5: begin Snd_Play(pfs_move); aIm_hero[3, 6].Visible := false; aIm_hero[3, 5].Visible := true; end; // gets ready to drink
    6: begin Snd_Play(pfs_move); aIm_hero[3, 5].Visible := false; aIm_hero[3, 6].Visible := true; end; // drinks
    7: begin Snd_Play(pfs_move); aIm_hero[3, 6].Visible := false; aIm_hero[3, 5].Visible := true; end; // gets ready to drink
    8: begin Snd_Play(pfs_move); aIm_hero[3, 5].Visible := false; aIm_hero[3, 6].Visible := true; Timer_Bonus.Interval := 1000; end; // drinks
    9: begin
         aIm_hero[3, 6].Visible := false;                                       // hides drink
         Vid_Show_Hero(iHeroPos, 0); iHeroPos := 2; Vid_Show_Hero(iHeroPos, 1); // places main character in iCharPox=2 and renders him
         iLoopTop := 5; Vid_Show_Item_Top(iLoopTop); iLoopTop := 4;             // renders "top" item and gets ready for next loop
         Gam_Set_Level;                                                         // sets game speed, difficulty etc.

         for i := 1 to 3 do                                                     // clears moveable items array
           begin
             for j := 1 to 6 do
               aItems[i, j] := 0;

             aItemsGot[i] := 0;                                                 // clears "got" items array
             aItemsMiss[i] := 0;                                                // clears "missed" items array
           end;

         Vid_Show_All_Item(false);                                              // clear items

         iLoop := 1;                                                            // initializes game loop
         bControllers := true;                                                  // enables main character control
         Timer_Game.Enabled := true;                                            // game reprises

         Timer_Bonus.Enabled := false;                                          // disables this timer
       end;
  end; // case..of

  Inc(iLoopBonus);
end;





procedure TForm1.Timer_ClockTimer(Sender: TObject);
{ transforms current time into "sprites" form, then displays it                }
  var
    present:              TDateTime;
    hour, min, sec, msec: Word;
begin
  present := Now;                                                               // gets system time...
  DecodeTime(present, hour, min, sec, msec);                                    // and decodes it

  if hour > 11 then                                                             // changes mode from 0/24 to 12 AM/12 PM
    begin
      Dec(hour, 12);
      im_panel_time_am.Visible := false;
      im_panel_time_pm.Visible := true;
    end
  else
    begin
      im_panel_time_am.Visible := true;
      im_panel_time_pm.Visible := false;
    end;

  if hour = 0 then                                                              // changes mode from 0/24 to 12 AM/12 PM
    hour := 12;

  if hour > 9 then                                                              // hour: tens
    aIm_panel_digit_pos[1].Picture := aIm_panel_number[hour div 10].Picture
  else
    aIm_panel_digit_pos[1].Picture := nil;

  aIm_panel_digit_pos[2].Picture := aIm_panel_number[hour mod 10].Picture;      // hour: units
  aIm_panel_digit_pos[3].Picture := aIm_panel_number[min div 10].Picture;       // minute: tens
  aIm_panel_digit_pos[4].Picture := aIm_panel_number[min mod 10].Picture;       // minute: units
  aIm_panel_digit_pos[5].Picture := nil;                                        // small digit

  Timer_Clock.Interval := 1000;
end;





procedure TForm1.Timer_DateTimer(Sender: TObject);
{ transforms current date into "sprites" form, then displays it                }
  var
    present:          TDateTime;
    day, month, year: Word;
begin
  present := Now;                                                               // gets system date...
  DecodeDate(present, year, month, day);                                        // and decodes it

  if month > 9 then                                                             // month: tens
    aIm_panel_digit_pos[1].Picture := aIm_panel_number[month div 10].Picture
  else
    aIm_panel_digit_pos[1].Picture := nil;

  aIm_panel_digit_pos[2].Picture := aIm_panel_number[month mod 10].Picture;     // month: units

  if day > 9 then                                                               // day: tens
    aIm_panel_digit_pos[3].Picture := aIm_panel_number[day div 10].Picture
  else
    aIm_panel_digit_pos[3].Picture := nil;

  aIm_panel_digit_pos[4].Picture := aIm_panel_number[day mod 10].Picture;       // day: units
  aIm_panel_digit_pos[5].Picture := nil;                                        // small digit

  Timer_Date.Interval := 1000;
end;





procedure TForm1.Timer_GameTimer(Sender: TObject);
{ main game action                                                             }
  var
    i, j,
    iOnscreen,
    iMistake,
    iMoved:    integer;
begin
  // inits local variables (for later use within this procedure)
  iMistake := 0;
  iMoved   := 0;

  // keeps and then removes "got" items
  if aItemsGot[iLoop] = 1 then
    aItemsGot[iLoop] := 0;

  if aItems[iLoop, 6] = 2 then
    begin
      aItems[iLoop, 6] := 0;
      aItemsGot[iLoop] := 1;
    end;

  // checks for mistakes
  for i := 1 to 3 do
    if (aItems[i, 6] = 1) and (iHeroPos <> i) then
      begin
        aItems[i, 6] := 0;
        aItemsMiss[i] := 1;
        iMistake := i;
      end;

  // checks for collisions character-items on certain positions and manages 'hit' animation of main character
  if (iMistake = 0) and (aItems[iHeroPos, 6] = 1) then
    begin
      bControllers := false;                                                    // stops player controls
      Vid_Show_Hero(iHeroPos, 2);                                               // displays player with raquet high

      iLoopGot := 1;
      Timer_Got.Interval := 50;
      Timer_Got.Enabled := true;                                                // enables "Timer_Got" timer

      aItems[iHeroPos, 6] := 2;                                                 // removed at next timer tick

      Timer_Game.Enabled := false;                                              // disables this timer
    end;

  // moves and generates items
  if iMistake = 0 then
    begin

      // moves items
      for i := 6 downto 2 do
        aItems[iLoop, i] := aItems[iLoop, i - 1];

      aItems[iLoop, 1] := 0;

      // generates new items
      iOnscreen := 0;

      for i := 1 to 3 do                                                        // counts items on screen
        for j := 1 to 6 do
          if aItems[i, j] = 1 then
            Inc(iOnscreen);

      if random(2) = 0 then aItems[iLoop, 1] := 1 else aItems[iLoop, 1] := 0;   // generates randomly
      if iOnscreen = 0 then aItems[iLoop, 1] := 1;                              // generates if none on screen
      if aItems[iLoop, 2] = 1 then aItems[iLoop, 1] := 0;                       // prevents from generating if one just appeared on the same column

      case iLoop of                                                             // prevents from generating if one just appeared on previous column
        1: if aItems[3, 1] = 1 then aItems[iLoop, 1] := 0;
        2: if aItems[1, 1] = 1 then aItems[iLoop, 1] := 0;
        3: if aItems[2, 1] = 1 then aItems[iLoop, 1] := 0;
      end;

      if iOnscreen >= iMaxOnscreen then aItems[iLoop, 1] := 0;                  // prevents from generating if more onscreen items than allowed

      // gets ready to play "pfMove" sound
      for i := 1 to 6 do
        if aItems[iLoop, i] = 1 then
          Inc(iMoved);
    end;

  // executes miss routine, counts misses and reprises or goes to "game over"
  if iMistake <> 0 then                                                         // checks if any miss just occured
    begin
      bControllers := false;
      Gam_Stop_All_Timers;
      aIm_item_miss[iMistake].visible := true;                                  // renders "missed" item on screen
      Inc(iMiss);                                                               // increases number of misses

      if iMiss < 3 then                                                         // checks how many misses were done
        begin
          iLoopMiss := 1;
          Timer_Reprise.Interval := 1;
          Timer_Reprise.Enabled := true;                                        // game continues if misses < 3
        end
      else
        begin
          iLoopMiss := 1;
          Timer_Game_Over.Interval := 1;
          Timer_Game_Over.Enabled := true;                                      // game is over if misses = 3
        end;
    end
  else
    begin                                                                       // no misses
      Inc(iLoop);                                                               // loops action

      if iLoop > 3 then                                                         
        iLoop := 1;                                             

      Inc(iLoopTop);                                                            // loops animation for items on top

      if iLooptop > 6 then
        iLoopTop := 3;                                    
    end;

  // renders moveable items on screen
  for i := 1 to 3 do
    for j := 1 to 6 do
      aIm_item[i, j].visible := (aItems[i, j] > 0);

  // renders 'got' items on screen
  for i := 1 to 3 do
    aIm_item_got[i].visible := (aItemsGot[i] = 1);

  aIm_hero[iLoop, 4].Visible := false;

  // renders 'top' items on screen
  Vid_Show_Item_Top(iLoopTop);

  // plays correct sound (miss or move)
  if iMistake > 0 then
    Snd_Play(pfs_miss)
  else
    if iMoved > 0 then
      Snd_Play(pfs_move);

  // calculates speed and difficulty for next timer tick
  Gam_Set_Level;
end;





procedure TForm1.Timer_Game_OverTimer(Sender: TObject);
{ game is over: play "Game Over" tune, then freeze for 30 seconds, and finally }
{ change mode to "Time 1", if no button is pressed                             }
  var
    i: integer;
begin
  case iLoopMiss of
    1: begin Snd_Play(pfs_miss);                               Timer_Game_Over.Interval := 140; end;
    2: begin Snd_Play(pfs_move); i := 1; Vid_Show_Item_Top(i); Timer_Game_Over.Interval := 375; end;
    3: begin Snd_Play(pfs_move); i := 2; Vid_Show_Item_Top(i); Timer_Game_Over.Interval := 500; end;
    4: begin Snd_Play(pfs_move); i := 1; Vid_Show_Item_Top(i); end;
    5: begin Snd_Play(pfs_move); i := 2; Vid_Show_Item_Top(i); end;
    6: begin Snd_Play(pfs_move); i := 1; Vid_Show_Item_Top(i); end;
    7: begin Snd_Play(pfs_move); i := 2; Vid_Show_Item_Top(i); Timer_Game_Over.Interval := 530; end;
    8: begin
         Vid_Show_Panel_Miss(iMiss);                                            // updates "misses" panel

         if bSoundIngame then
           begin
             Snd_Play(pfs_gameover);                                            // plays "Game over" tune

             Timer_Game_Over.Interval := 4200;
           end
         else
           Timer_Game_Over.Interval := 100;
       end;
    9: begin
         if (iMode = 8) and (iScore > iHiScoreA) then                           // checks if final score is new hiscore (Game A)
           begin
             iHiScoreA := iScore;
             iReg.WriteInteger('', 'HiScore1', iHiScoreA);                      // saves to system registry
           end;
         if (iMode = 9) and (iScore>iHiScoreB) then                             // checks if final score is new hiscore (Game B)
           begin
             iHiScoreB := iScore;
             iReg.WriteInteger('', 'HiScore2', iHiScoreB);                      // saves to system registry
           end;

         iMode := 10; Gam_Set_Mode;                                             // sets "Game over" mode

         Timer_Game_Over.Interval := 30000;
       end;
    10: begin
         iMode := 1; Gam_Set_Mode;                                              // waits 30 seconds, then goes into "Time 1" mode automatically

         Timer_Game_Over.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopMiss);
end;





procedure TForm1.Timer_Game_StartTimer(Sender: TObject);
{ start music ends, and gane actually begins                                   }
begin
  Vid_Show_Panel_Score(iScore, true);                                           // displays iScore = 0 now
  bControllers := true;                                                         // enables main character control

  iLoop := 1;                                                                   // initializes game loop
  Timer_Game.Enabled := true;                                                   // game starts here

  Timer_Game_Start.Enabled := false;                                            // disables this timer
end;





procedure TForm1.Timer_GotTimer(Sender: TObject);
{ main character goes back to "normal" animaton frame                          }
  var
    i: integer;
begin
  case iLoopGot of
  1: begin
       Vid_Show_Hero(iHeroPos, 2);
       Snd_Play(pfs_got);
       Inc(iScore, 10);                                                         // increases score

       if (iScore mod 1000 = 0) then                                            // checks if score=1000, 2000, 3000, etc.
         begin                                                                    // bonus scene
           bControllers := false;
           Gam_Stop_All_Timers;

           iLoopBonus := iHeroPos;                                                // initializes "bonus" loop, starting at iLoopBonus=1, 2 or 3
           Timer_Bonus.Interval := 375;
           Timer_Bonus.Enabled := true;
         end
       else                                                                     // score is not multiple of 1000
         begin
           aIm_hero[iHeroPos, 4].visible := true;                                 // hides "got" sprite
           bControllers := true;                                                  // enables player controls again

           Timer_Game.Enabled := true;                                            // game reprises

           Timer_Got.Interval := 100;                                             // initializes next timer tick
         end;

       if iScore > 9999 then                                                    // resets score to zero if above 9999
         Dec(iScore, 10000);                                   

       Vid_Show_Panel_Score(iScore, true);                                      // renders score
     end;
  2: begin
       for i := 1 to 3 do                                                       // renders main char with raquet down
         if i = iHeroPos then
           Vid_Show_Hero(i, 1)
         else
           Vid_Show_Hero(i, 0);

       Timer_Got.Enabled := false;                                                // disables this timer
     end;
  end; // case..of

  Inc(iLoopGot);
end;





procedure TForm1.Timer_RepriseTimer(Sender: TObject);
{ game continues after "miss"                                                  }
  var
    i: integer;
begin
  case iLoopMiss of
    1: begin Snd_Play(pfs_miss);                                Timer_Reprise.Interval := 140; end;
    2: begin Snd_Play(pfs_move); i := 7;  Vid_Show_Item_Top(i); Timer_Reprise.Interval := 375; end;
    3: begin Snd_Play(pfs_move); i := 8;  Vid_Show_Item_Top(i); Timer_Reprise.Interval := 500; end;
    4: begin Snd_Play(pfs_move); i := 7;  Vid_Show_Item_Top(i); end;
    5: begin Snd_Play(pfs_move); i := 9;  Vid_Show_Item_Top(i); end;
    6: begin Snd_Play(pfs_move); i := 7;  Vid_Show_Item_Top(i); end;
    7: begin Snd_Play(pfs_move); i := 10; Vid_Show_Item_Top(i); end;
    8: begin
         Vid_Show_Panel_Miss(iMiss);                                            // updates "misses" panel
         iLoopTop := 5; Vid_Show_Item_Top(iLoopTop); iLoopTop := 4;             // renders "top" item and gets ready for next loop

         for i := 1 to 3 do
           aItemsMiss[i] := 0;

         Vid_Show_All_Item_Miss(false);                                         // clear "missed" items
         bControllers := true;

         Timer_Game.Enabled := true;

         Timer_Reprise.Enabled := false;                                        // game reprises
       end;
  end; // case..of

  Inc(iLoopMiss);
end;





procedure TForm1.Timer_StartupTimer(Sender: TObject);
{ goes into "Time 1" mode , 30 seconds after game is turned on                 }
begin
  iMode := 1;
  Gam_Set_Mode;

  Timer_Startup.Enabled := false;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.pm_0_menuPopup(Sender: TObject);
{ shows the "checked" menuitems if needed. also, hides the game menu if it is  }
{ visible when the popup menu pops up                                          }
begin
{}pm_11_splashscreen.Checked := iReg.ReadBool('', 'SplashScreen', SplashForm.chb_splashscreen.Checked);

{}if bFullscreenMode
    and bGameMenuOpen then
    begin
      bGameMenuOpen := false;
      Form1.Vid_Display_Game_Menu(bGameMenuOpen);
    end;
end;





procedure TForm1.pm_1_aboutClick(Sender: TObject);
{ displays the embedded "About the author..." form                             }
begin
  AboutForm.Show;
end;





procedure TForm1.pm_2_readmeClick(Sender: TObject);
{ opens the "Readme.txt" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Readme.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_3_whatsnewClick(Sender: TObject);
{ opens the "Whatsnew.txt" file                                                }
begin
  ShellExecute(Handle, 'open', 'Whatsnew.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_4_manualClick(Sender: TObject);
{ displays the embedded instructions manual                                    }
begin
  ManualForm.Show;
end;





procedure TForm1.pm_6_configClick(Sender: TObject);
{ opens the "Config.ini" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Config.ini', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_7_keysClick(Sender: TObject);
{ displays the "key settings" form                                             }
begin
  KeysForm.position := poScreenCenter;
  KeysForm.visible := true;
  Form1.Enabled := false;
end;





procedure TForm1.pm_8_hiscoreClick(Sender: TObject);
{ resets high scores to the system registry                                    }
begin
  iHiScoreA := 0; iReg.WriteInteger('', 'HiScore1', iHiScoreA);
  iHiScoreB := 0; iReg.WriteInteger('', 'HiScore2', iHiScoreB);
  Beep;
end;





{}procedure TForm1.pm_10_fullscreenClick(Sender: TObject);
{ changes from fullscreen to windowed, and vice-versa                          }
begin
  // updates popup menu
  bFullscreenMode := not bFullscreenMode;
  pm_10_fullscreen.Checked := bFullscreenMode;
  bGameMenuOpen := false;

  iReg.WriteBool('', 'FullscreenMode', bFullscreenMode);

  // graphics init
  Top  := - Screen.Height;
  Left := - Screen.Width;
  Gfx_Game_Init;

{}if bFullscreenMode then
{}  Position := poScreenCenter
{}else
{}  begin
{}    // centers horizontally only and sets Y=0
{}    Top := 0;
{}    Left := (Screen.Width - Form1.ClientWidth) div 2;
{}  end;
end;





procedure TForm1.pm_11_splashscreenClick(Sender: TObject);
{ updates the ticks to the checkbox, the popup menu and the system registry    }
begin
{}pm_11_splashscreen.Checked := not pm_11_splashscreen.Checked;

{}iReg.WriteBool('', 'SplashScreen', pm_11_splashscreen.Checked);
end;





procedure TForm1.pm_14_exitClick(Sender: TObject);
{ exits the game                                                               }
begin
  Application.Terminate;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.btn_game_menu_topClick(Sender: TObject);
{ sets the game menu as visible/hidden, then displays/hides it                 }
begin
  bGameMenuOpen := not bGameMenuOpen;
  Vid_Display_Game_Menu(bGameMenuOpen);
end;





procedure TForm1.la_menu_itemMouseEnter(Sender: TObject);
{ the game menu label bg colour becomes solid                                  }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := false;
end;





procedure TForm1.la_menu_itemMouseLeave(Sender: TObject);
{ the game menu label bg colour becomes transparent                            }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := true;
end;





procedure TForm1.la_menu_itemMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ hides the game menu, once a label has been pressed                           }
begin
  if Button = mbLeft then
    begin
      bGameMenuOpen := false;
      Vid_Display_Game_Menu(false);
    end;
end;





procedure TForm1.Vid_Display_Game_Menu(b: boolean);
{ displays/hides the game menu, depending on the value of bGameMenuOpen        }
  var
    i: integer;
begin
  case bGameMenuOpen of
    true: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i] := TLabel.Create(Form1);
            aMenuLabels[i].Parent := Form1;

            aMenuLabels[i].Top := 32 * i;
            aMenuLabels[i].Left := 0;

            aMenuLabels[i].AutoSize := false;
            aMenuLabels[i].Height := 32;
            aMenuLabels[i].Width := 256;
            aMenuLabels[i].Layout := tlCenter;

            aMenuLabels[i].Transparent := true;

            if aMenuItem[i].Caption <> '-' then
              begin
                aMenuLabels[i].Font.Name := 'Arial';
                aMenuLabels[i].Font.Color := clWhite;
                aMenuLabels[i].Font.Height := 16;
                aMenuLabels[i].Caption := '   ' + aMenuItem[i].Caption;

                if aMenuItem[i].Checked then
                  aMenuLabels[i].Caption := aMenuLabels[i].Caption + ' [V]';

                aMenuLabels[i].Color := $00000000;
                aMenuLabels[i].Cursor := crHandPoint;
                aMenuLabels[i].Enabled := aMenuItem[i].Enabled;

                if aMenuLabels[i].Enabled then
                  begin
                    aMenuLabels[i].OnClick      := aMenuItem[i].OnClick;
                    aMenuLabels[i].OnMouseEnter := la_menu_itemMouseEnter;
                    aMenuLabels[i].OnMouseLeave := la_menu_itemMouseLeave;
                    aMenuLabels[i].OnMouseUp    := la_menu_itemMouseUp;
                  end;
              end;
          end;

        btn_game_menu_background.Width := 256;
        btn_game_menu_background.Height := 32 * (MENU_ITEMS + 1)
      end;
    false: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i].Free;
          end;

        btn_game_menu_background.Height := 32;
        btn_game_menu_background.Width := 36;
      end;
    end; // case..of
end;





initialization
  bStartup := true;





end.
